package com.mdd.admin.service.impl;

import cn.dev33.satoken.stp.StpUtil;
import com.alibaba.fastjson2.JSONObject;
import com.alibaba.fastjson2.JSONWriter;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Assert;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.yulichang.query.MPJQueryWrapper;
import com.mdd.admin.service.ISystemAuthAdminService;
import com.mdd.admin.service.ISystemAuthPermService;
import com.mdd.admin.socket.client.WebSocketService;
import com.mdd.admin.validate.commons.PageValidate;
import com.mdd.admin.validate.system.SystemAdminCreateValidate;
import com.mdd.admin.validate.system.SystemAdminSearchValidate;
import com.mdd.admin.validate.system.SystemAdminUpInfoValidate;
import com.mdd.admin.validate.system.SystemAdminUpdateValidate;
import com.mdd.admin.validate.user.UserLogoutValidate;
import com.mdd.admin.vo.system.SystemAuthAdminDetailVo;
import com.mdd.admin.vo.system.SystemAuthAdminInformVo;
import com.mdd.admin.vo.system.SystemAuthAdminListedVo;
import com.mdd.admin.vo.system.SystemAuthAdminSelvesVo;
import com.mdd.common.core.AjaxResult;
import com.mdd.common.core.PageResult;
import com.mdd.common.entity.License;
import com.mdd.common.entity.system.SystemAuthAdmin;
import com.mdd.common.entity.system.SystemAuthDept;
import com.mdd.common.entity.system.SystemAuthMenu;
import com.mdd.common.entity.system.SystemAuthRole;
import com.mdd.common.exception.OperateException;
import com.mdd.common.mapper.LicenseMapper;
import com.mdd.common.mapper.system.SystemAuthAdminMapper;
import com.mdd.common.mapper.system.SystemAuthDeptMapper;
import com.mdd.common.mapper.system.SystemAuthMenuMapper;
import com.mdd.common.mapper.system.SystemAuthRoleMapper;
import com.mdd.common.util.ListUtils;
import com.mdd.common.util.StringUtils;
import com.mdd.common.util.TimeUtils;
import com.mdd.common.util.ToolUtils;
import com.mdd.common.util.UrlUtils;
import java.util.Arrays;
import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Objects;
import javax.annotation.Resource;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

@Service
public class SystemAuthAdminServiceImpl implements ISystemAuthAdminService {
   @Resource
   SystemAuthAdminMapper systemAuthAdminMapper;
   @Resource
   SystemAuthMenuMapper systemAuthMenuMapper;
   @Resource
   SystemAuthDeptMapper systemAuthDeptMapper;
   @Resource
   SystemAuthRoleMapper systemAuthRoleMapper;
   @Resource
   ISystemAuthPermService iSystemAuthPermService;
   @Resource
   LicenseMapper licenseMapper;
   @Resource
   WebSocketService socketService;

   public PageResult<SystemAuthAdminListedVo> list(PageValidate pageValidate, SystemAdminSearchValidate searchValidate) {
      Integer page = pageValidate.getPageNo();
      Integer limit = pageValidate.getPageSize();
      MPJQueryWrapper<SystemAuthAdmin> mpjQueryWrapper = new MPJQueryWrapper();
      ((MPJQueryWrapper)mpjQueryWrapper.select(new String[]{"t.id,t.username,t.nickname,t.avatar,t.goal, t.count, t.card_num as cardNum, t.card_only as cardOnly, t.loop_level as loopLevel,t.role_ids as role,t.dept_ids as dept,t.is_multipoint,t.is_disable,t.last_login_ip,t.last_login_time,t.create_time,t.update_time"}).eq("t.is_delete", 0)).orderByDesc(Arrays.asList("t.loop_level", "t.id"));
      this.systemAuthAdminMapper.setSearch(mpjQueryWrapper, searchValidate, new String[]{"like:username:str", "like:nickname:str"});
      if (StringUtils.isNotNull(searchValidate.getRole())) {
         mpjQueryWrapper.in("role_ids", Collections.singletonList(searchValidate.getRole()));
      }

      IPage<SystemAuthAdminListedVo> iPage = this.systemAuthAdminMapper.selectJoinPage(new Page((long)page, (long)limit), SystemAuthAdminListedVo.class, mpjQueryWrapper);
      Iterator var7 = iPage.getRecords().iterator();

      while(var7.hasNext()) {
         SystemAuthAdminListedVo vo = (SystemAuthAdminListedVo)var7.next();
         LinkedList dept;
         List deptIds;
         List deptList;
         Iterator var12;
         if (vo.getId().equals(1)) {
            vo.setRole("系统管理员");
         } else {
            dept = new LinkedList();
            deptIds = ListUtils.stringToListAsInt(vo.getRole(), ",");
            if (!deptIds.isEmpty()) {
               deptList = this.systemAuthRoleMapper.selectList((Wrapper)(new QueryWrapper()).select(new String[]{"id,name"}).in("id", deptIds));
               var12 = deptList.iterator();

               while(var12.hasNext()) {
                  SystemAuthRole d = (SystemAuthRole)var12.next();
                  dept.add(d.getName());
               }
            }

            vo.setRole(ListUtils.listToStringByStr(dept, ","));
         }

         if (!StringUtils.isNull(vo.getDept()) && !vo.getDept().equals("")) {
            dept = new LinkedList();
            deptIds = ListUtils.stringToListAsInt(vo.getDept(), ",");
            if (!deptIds.isEmpty()) {
               deptList = this.systemAuthDeptMapper.selectList((Wrapper)((QueryWrapper)(new QueryWrapper()).select(new String[]{"id,name"}).in("id", deptIds)).eq("is_delete", 0));
               var12 = deptList.iterator();

               while(var12.hasNext()) {
                  SystemAuthDept d = (SystemAuthDept)var12.next();
                  dept.add(d.getName());
               }
            }

            vo.setDept(ListUtils.listToStringByStr(dept, ","));
         } else {
            vo.setDept("");
         }

         vo.setAvatar(UrlUtils.toAbsoluteUrl(vo.getAvatar()));
         vo.setCreateTime(TimeUtils.timestampToDate(vo.getCreateTime()));
         vo.setUpdateTime(TimeUtils.timestampToDate(vo.getUpdateTime()));
         vo.setLastLoginTime(TimeUtils.timestampToDate(vo.getLastLoginTime()));
      }

      return PageResult.iPageHandle(iPage);
   }

   public SystemAuthAdminSelvesVo self(Integer adminId) {
      SystemAuthAdmin sysAdmin = (SystemAuthAdmin)this.systemAuthAdminMapper.selectOne((Wrapper)((QueryWrapper)((QueryWrapper)(new QueryWrapper()).select(SystemAuthAdmin.class, (info) -> {
         return !info.getColumn().equals("salt") && !info.getColumn().equals("password") && !info.getColumn().equals("is_delete") && !info.getColumn().equals("delete_time");
      }).eq("is_delete", 0)).eq("id", adminId)).last("limit 1"));
      SystemAuthAdminInformVo systemAuthAdminInformVo = new SystemAuthAdminInformVo();
      BeanUtils.copyProperties(sysAdmin, systemAuthAdminInformVo);
      systemAuthAdminInformVo.setAvatar(UrlUtils.toAbsoluteUrl(sysAdmin.getAvatar()));
      systemAuthAdminInformVo.setUpdateTime(TimeUtils.timestampToDate(sysAdmin.getUpdateTime()));
      systemAuthAdminInformVo.setCreateTime(TimeUtils.timestampToDate(sysAdmin.getCreateTime()));
      systemAuthAdminInformVo.setLastLoginTime(TimeUtils.timestampToDate(sysAdmin.getLastLoginTime()));
      JSONObject perms = new JSONObject();
      if (StringUtils.isNotEmpty(JSONObject.parseObject(sysAdmin.getPerms()))) {
         try {
            perms = JSONObject.parseObject(sysAdmin.getPerms());
         } catch (Exception var12) {
         }
      }

      List<String> auths = new LinkedList();
      if (adminId > 1) {
         List<Integer> roleIds = ListUtils.stringToListAsInt(sysAdmin.getRoleIds(), ",");
         List<Integer> menuIds = this.iSystemAuthPermService.selectMenuIdsByRoleId(roleIds);
         if (menuIds.size() > 0) {
            List<SystemAuthMenu> systemAuthMenus = this.systemAuthMenuMapper.selectList((Wrapper)((QueryWrapper)((QueryWrapper)((QueryWrapper)(new QueryWrapper()).eq("is_disable", 0)).in("id", menuIds)).in("menu_type", Arrays.asList("C", "A"))).orderByAsc(Arrays.asList("menu_sort", "id")));
            Iterator var9 = systemAuthMenus.iterator();

            label47:
            while(true) {
               String permsKey;
               do {
                  do {
                     do {
                        if (!var9.hasNext()) {
                           break label47;
                        }

                        SystemAuthMenu item = (SystemAuthMenu)var9.next();
                        permsKey = item.getPerms();
                     } while(!StringUtils.isNotNull(permsKey));
                  } while(!StringUtils.isNotEmpty(permsKey));

                  permsKey = permsKey.trim();
               } while(perms.containsKey(permsKey) && !Objects.equals(perms.getInteger(permsKey), 1));

               auths.add(permsKey);
            }
         }

         if (auths.size() <= 0) {
            auths.add("");
         }
      } else {
         auths.add("*");
      }

      SystemAuthAdminSelvesVo vo = new SystemAuthAdminSelvesVo();
      vo.setUser(systemAuthAdminInformVo);
      vo.setPermissions(auths);
      return vo;
   }

   public SystemAuthAdminDetailVo detail(Integer id) {
      SystemAuthAdmin sysAdmin = (SystemAuthAdmin)this.systemAuthAdminMapper.selectOne((Wrapper)((QueryWrapper)((QueryWrapper)(new QueryWrapper()).select(SystemAuthAdmin.class, (info) -> {
         return !info.getColumn().equals("salt") && !info.getColumn().equals("password") && !info.getColumn().equals("is_delete") && !info.getColumn().equals("delete_time");
      }).eq("id", id)).eq("is_delete", 0)).last("limit 1"));
      Assert.notNull(sysAdmin, "账号已不存在！", new Object[0]);
      SystemAuthAdminDetailVo vo = new SystemAuthAdminDetailVo();
      BeanUtils.copyProperties(sysAdmin, vo);
      vo.setRoleIds(Integer.parseInt(sysAdmin.getRoleIds()));
      vo.setAvatar(UrlUtils.toAbsoluteUrl(sysAdmin.getAvatar()));
      vo.setCreateTime(TimeUtils.timestampToDate(sysAdmin.getCreateTime()));
      vo.setUpdateTime(TimeUtils.timestampToDate(sysAdmin.getUpdateTime()));
      vo.setLastLoginTime(TimeUtils.timestampToDate(sysAdmin.getLastLoginTime()));
      return vo;
   }

   public void add(SystemAdminCreateValidate createValidate) {
      String[] field = new String[]{"id", "username", "nickname"};
      Assert.isNull(this.systemAuthAdminMapper.selectOne((Wrapper)((QueryWrapper)((QueryWrapper)(new QueryWrapper()).select(field).eq("is_delete", 0)).eq("username", createValidate.getUsername())).last("limit 1")), "账号已存在换一个吧！", new Object[0]);
      String salt = ToolUtils.randomString(5);
      String pwd = ToolUtils.makeMd5(createValidate.getPassword().trim() + salt);
      String createAvatar = createValidate.getAvatar();
      String defaultAvatar = "/api/static/backend_avatar.png";
      String avatar = StringUtils.isNotEmpty(createValidate.getAvatar()) ? UrlUtils.toRelativeUrl(createAvatar) : defaultAvatar;
      SystemAuthAdmin model = new SystemAuthAdmin();
      model.setRoleIds(createValidate.getRoleIds() + "");
      model.setDeptIds("");
      model.setPostIds("");
      model.setUsername(createValidate.getUsername());
      model.setNickname(createValidate.getUsername());
      model.setAvatar(avatar);
      model.setPassword(pwd);
      model.setSalt(salt);
      model.setSort(createValidate.getSort());
      model.setIsMultipoint(0);
      model.setPerms(createValidate.getRoleIds() == 2 ? "{\"kzfw.actions\":0}" : createValidate.getPerms());
      model.setGoal(createValidate.getGoal());
      model.setLoopLevel(createValidate.getLoopLevel());
      model.setIsDisable(createValidate.getIsDisable());
      model.setCreateTime(System.currentTimeMillis() / 1000L);
      model.setUpdateTime(System.currentTimeMillis() / 1000L);
      this.systemAuthAdminMapper.insert(model);
   }

   public void edit(SystemAdminUpdateValidate updateValidate, Integer adminId) {
      if (!adminId.equals(1) && updateValidate.getId().equals(1)) {
         throw new OperateException("您无权限编辑系统管理员!");
      } else {
         String[] field = new String[]{"id", "username", "nickname"};
         Assert.notNull(this.systemAuthAdminMapper.selectOne((Wrapper)((QueryWrapper)((QueryWrapper)(new QueryWrapper()).select(field).eq("id", updateValidate.getId())).eq("is_delete", 0)).last("limit 1")), "账号不存在了!", new Object[0]);
         Assert.isNull(this.systemAuthAdminMapper.selectOne((Wrapper)((QueryWrapper)((QueryWrapper)((QueryWrapper)(new QueryWrapper()).select(field).eq("is_delete", 0)).eq("username", updateValidate.getUsername())).ne("id", updateValidate.getId())).last("limit 1")), "账号已存在换一个吧!", new Object[0]);
         Assert.isNull(this.systemAuthAdminMapper.selectOne((Wrapper)((QueryWrapper)((QueryWrapper)((QueryWrapper)(new QueryWrapper()).select(field).eq("is_delete", 0)).eq("nickname", updateValidate.getNickname())).ne("id", updateValidate.getId())).last("limit 1")), "昵称已存在换一个吧!", new Object[0]);
         SystemAuthAdmin model = new SystemAuthAdmin();
         model.setId(updateValidate.getId());
         model.setRoleIds(updateValidate.getRoleIds() + "");
         model.setDeptIds("");
         model.setPostIds("");
         model.setNickname(updateValidate.getUsername());
         model.setAvatar(UrlUtils.toRelativeUrl(updateValidate.getAvatar()));
         model.setSort(updateValidate.getSort());
         model.setIsMultipoint(0);
         model.setPerms(updateValidate.getRoleIds() == 2 ? "{\"kzfw.actions\":0}" : updateValidate.getPerms());
         model.setGoal(updateValidate.getGoal());
         model.setLoopLevel(updateValidate.getLoopLevel());
         model.setIsDisable(updateValidate.getIsDisable());
         model.setUpdateTime(System.currentTimeMillis() / 1000L);
         if (!updateValidate.getId().equals(1)) {
            model.setUsername(updateValidate.getUsername());
         }

         if (StringUtils.isNotNull(updateValidate.getPassword()) && StringUtils.isNotEmpty(updateValidate.getPassword())) {
            String salt = ToolUtils.randomString(5);
            String pwd = ToolUtils.makeMd5(updateValidate.getPassword().trim() + salt);
            model.setPassword(pwd);
            model.setSalt(salt);
         }

         this.systemAuthAdminMapper.updateById(model);
         if (StringUtils.isNotNull(updateValidate.getPassword()) && StringUtils.isNotEmpty(updateValidate.getPassword())) {
            StpUtil.kickout(updateValidate.getId());
         }

      }
   }

   public void resetCount(Integer adminId) {
      SystemAuthAdmin model = new SystemAuthAdmin();
      model.setId(adminId);
      model.setCount(0);
      model.setCardNum(0);
      model.setCardOnly(0);
      this.systemAuthAdminMapper.updateById(model);
   }

   public void upInfo(SystemAdminUpInfoValidate upInfoValidate, Integer adminId) {
      SystemAuthAdmin model = (SystemAuthAdmin)this.systemAuthAdminMapper.selectOne((Wrapper)((QueryWrapper)((QueryWrapper)(new QueryWrapper()).select(new String[]{"id,username,nickname,password,salt"}).eq("id", adminId)).eq("is_delete", 0)).last("limit 1"));
      Assert.notNull(model, "账号不存在了!", new Object[0]);
      String createAvatar = upInfoValidate.getAvatar();
      String defaultAvatar = "/api/static/backend_avatar.png";
      String avatar = StringUtils.isNotEmpty(upInfoValidate.getAvatar()) ? UrlUtils.toRelativeUrl(createAvatar) : defaultAvatar;
      model.setAvatar(avatar);
      model.setUpdateTime(System.currentTimeMillis() / 1000L);
      if (StringUtils.isNotNull(upInfoValidate.getPassword()) && StringUtils.isNotEmpty(upInfoValidate.getPassword())) {
         String currPassword = ToolUtils.makeMd5(upInfoValidate.getCurrPassword() + model.getSalt());
         Assert.isFalse(!currPassword.equals(model.getPassword()), "当前密码不正确!", new Object[0]);
         String salt = ToolUtils.randomString(5);
         String pwd = ToolUtils.makeMd5(upInfoValidate.getPassword().trim() + salt);
         model.setPassword(pwd);
         model.setSalt(salt);
      }

      this.systemAuthAdminMapper.updateById(model);
      if (StringUtils.isNotNull(upInfoValidate.getPassword()) && StringUtils.isNotEmpty(upInfoValidate.getPassword())) {
         StpUtil.kickout(adminId);
      }

   }

   public void del(Integer id, Integer adminId) {
      String[] field = new String[]{"id", "username", "nickname"};
      Assert.notNull(this.systemAuthAdminMapper.selectOne((Wrapper)((QueryWrapper)((QueryWrapper)(new QueryWrapper()).select(field).eq("id", id)).eq("is_delete", 0)).last("limit 1")), "账号已不存在!", new Object[0]);
      Assert.isFalse(id.equals(1), "系统管理员不允许删除!", new Object[0]);
      Assert.isFalse(id.equals(adminId), "不能删除自己!", new Object[0]);
      SystemAuthAdmin model = new SystemAuthAdmin();
      model.setId(id);
      model.setIsDelete(1);
      model.setDeleteTime(System.currentTimeMillis() / 1000L);
      this.systemAuthAdminMapper.updateById(model);
      StpUtil.kickout(id);
   }

   public void disable(Integer id, Integer adminId) {
      SystemAuthAdmin systemAuthAdmin = (SystemAuthAdmin)this.systemAuthAdminMapper.selectOne((Wrapper)((QueryWrapper)((QueryWrapper)(new QueryWrapper()).select(new String[]{"id,username,nickname,is_disable"}).eq("id", id)).eq("is_delete", 0)).last("limit 1"));
      Assert.notNull(systemAuthAdmin, "账号已不存在!", new Object[0]);
      Assert.isFalse(id.equals(adminId), "不能禁用自己!", new Object[0]);
      Integer disable = systemAuthAdmin.getIsDisable() == 1 ? 0 : 1;
      systemAuthAdmin.setIsDisable(disable);
      systemAuthAdmin.setUpdateTime(TimeUtils.timestamp());
      this.systemAuthAdminMapper.updateById(systemAuthAdmin);
      if (disable.equals(1)) {
         StpUtil.kickout(id);
      }

   }

   public AjaxResult<Object> logout(UserLogoutValidate logoutValidate) {
      QueryWrapper<License> queryWrapper = new QueryWrapper();
      queryWrapper.orderByDesc("license_id");
      IPage<License> iPage = this.licenseMapper.selectPage(new Page(1L, 1L), queryWrapper);
      if (iPage.getRecords() != null && !iPage.getRecords().isEmpty()) {
         License license = (License)iPage.getRecords().get(0);
         if (!Objects.equals(license.getCode(), logoutValidate.getLicense())) {
            return AjaxResult.failed("license 无效");
         } else {
            SystemAuthAdmin sysAdmin = (SystemAuthAdmin)this.systemAuthAdminMapper.selectOne((Wrapper)((QueryWrapper)(new QueryWrapper()).eq("username", logoutValidate.getUserName())).last("limit 1"));
            if (sysAdmin == null) {
               return AjaxResult.failed("账号不存在");
            } else {
               StpUtil.logout(sysAdmin.getId());
               JSONObject json = new JSONObject();
               json.put("option", "logout");
               json.put("adminId", sysAdmin.getId());
               this.socketService.sendMessage(json.toJSONString(new JSONWriter.Feature[0]));
               return null;
            }
         }
      } else {
         return AjaxResult.failed("未查询到license");
      }
   }
}
